@@ -50,13 +50,33 @@ certificateChecks checks x509s = do
 
 {- on windows and OSX, the trusted certificates are not yet accessible,
  - for now, print a big fat warning (better than nothing) and returns true  -}
-certificateVerifyChain :: [X509] -> IO TLSCertificateUsage
-certificateVerifyChain _ = do
+certificateVerifyChain_ :: [X509] -> IO TLSCertificateUsage
+certificateVerifyChain_ _ = do
 	putStrLn "****************** certificate verify chain doesn't yet work on your platform **********************"
 	putStrLn "please consider contributing to the certificate package to fix this issue"
 	return CertificateUsageAccept
 
 #else
+certificateVerifyChain_ :: [X509] -> IO TLSCertificateUsage
+certificateVerifyChain_ []     = return $ CertificateUsageReject (CertificateRejectOther "empty chain / no certificates")
+certificateVerifyChain_ (x:xs) = do
+	-- find a matching certificate that we trust (== installed on the system)
+	foundCert <- SysCert.findCertificate (certMatchDN x)
+	case foundCert of
+		Just sysx509 -> do
+			validChain <- certificateVerifyAgainst x sysx509
+			if validChain
+				then return CertificateUsageAccept
+				else return $ CertificateUsageReject (CertificateRejectOther "chain doesn't match each other")
+		Nothing      -> case xs of
+			[] -> return $ CertificateUsageReject CertificateRejectUnknownCA
+			_  -> do
+				validChain <- certificateVerifyAgainst x (head xs)
+				if validChain
+					then certificateVerifyChain_ xs
+					else return $ CertificateUsageReject (CertificateRejectOther "chain doesn't match each other")
+#endif
+
 -- | verify a certificates chain using the system certificates available.
 --
 -- each certificate of the list is verified against the next certificate, until
@@ -72,24 +92,13 @@ certificateVerifyChain _ = do
 -- TODO: verify validity, check revocation list if any, add optional user output to know
 -- the rejection reason.
 certificateVerifyChain :: [X509] -> IO TLSCertificateUsage
-certificateVerifyChain []     = return $ CertificateUsageReject (CertificateRejectOther "empty chain / no certificates")
-certificateVerifyChain (x:xs) = do
-	-- find a matching certificate that we trust (== installed on the system)
-	foundCert <- SysCert.findCertificate (certMatchDN x)
-	case foundCert of
-		Just sysx509 -> do
-			validChain <- certificateVerifyAgainst x sysx509
-			if validChain
-				then return CertificateUsageAccept
-				else return $ CertificateUsageReject (CertificateRejectOther "chain doesn't match each other")
-		Nothing      -> case xs of
-			[] -> return $ CertificateUsageReject CertificateRejectUnknownCA
-			_  -> do
-				validChain <- certificateVerifyAgainst x (head xs)
-				if validChain
-					then certificateVerifyChain xs
-					else return $ CertificateUsageReject (CertificateRejectOther "chain doesn't match each other")
-#endif
+certificateVerifyChain = certificateVerifyChain_ . reorderList
+	where
+		reorderList []     = []
+		reorderList (x:xs) =
+			case find (certMatchDN x) xs of
+				Nothing    -> x : reorderList xs
+				Just found -> x : found : reorderList (filter (/= found) xs)
 
 -- | verify a certificate against another one.
 -- the first certificate need to be signed by the second one for this function to succeed.
@@ -121,17 +130,16 @@ verifyF :: SignatureALG -> PubKey -> B.ByteString -> B.ByteString -> Either Stri
 --   md2WithRSAEncryption OBJECT IDENTIFIER ::= { pkcs-1 2 }
 --   md4WithRSAEncryption OBJECT IDENTIFIER ::= { pkcs-1 3 }
 --   md5WithRSAEncryption OBJECT IDENTIFIER ::= { pkcs-1 4 }
-verifyF (SignatureALG HashMD2 PubKeyALG_RSA) (PubKeyRSA rsak) = rsaVerify MD2.hash asn1 (mkRSA rsak)
+verifyF (SignatureALG HashMD2 PubKeyALG_RSA) (PubKeyRSA rsak) = rsaVerify MD2.hash asn1 rsak
 	where asn1 = "\x30\x20\x30\x0c\x06\x08\x2a\x86\x48\x86\xf7\x0d\x02\x05\x05\x00\x02\x10"
 
-verifyF (SignatureALG HashMD5 PubKeyALG_RSA) (PubKeyRSA rsak) = rsaVerify MD5.hash asn1 (mkRSA rsak)
+verifyF (SignatureALG HashMD5 PubKeyALG_RSA) (PubKeyRSA rsak) = rsaVerify MD5.hash asn1 rsak
 	where asn1 = "\x30\x20\x30\x0c\x06\x08\x2a\x86\x48\x86\xf7\x0d\x02\x05\x05\x00\x04\x10"
 
-verifyF (SignatureALG HashSHA1 PubKeyALG_RSA) (PubKeyRSA rsak) = rsaVerify SHA1.hash asn1 (mkRSA rsak)
+verifyF (SignatureALG HashSHA1 PubKeyALG_RSA) (PubKeyRSA rsak) = rsaVerify SHA1.hash asn1 rsak
 	where asn1 = "\x30\x21\x30\x09\x06\x05\x2b\x0e\x03\x02\x1a\x05\x00\x04\x14"
 
-verifyF (SignatureALG HashSHA1 PubKeyALG_DSA) (PubKeyDSA (pub,p,q,g)) = dsaSHA1Verify pk
-	where pk = DSA.PublicKey { DSA.public_params = (p,g,q), DSA.public_y = pub }
+verifyF (SignatureALG HashSHA1 PubKeyALG_DSA) (PubKeyDSA dsak) = dsaSHA1Verify dsak
 			
 verifyF _ _ = (\_ _ -> Left "unexpected/wrong signature")
 
@@ -140,9 +148,6 @@ dsaSHA1Verify pk _ b = either (Left . show) (Right) $ DSA.verify asig SHA1.hash
 
 rsaVerify h hdesc pk a b = either (Left . show) (Right) $ RSA.verify h hdesc pk a b
 
-mkRSA (lenmodulus, modulus, e) =
-	RSA.PublicKey { RSA.public_sz = lenmodulus, RSA.public_n = modulus, RSA.public_e = e }
-
 -- | Verify that the given certificate chain is application to the given fully qualified host name.
 certificateVerifyDomain :: String -> [X509] -> TLSCertificateUsage
 certificateVerifyDomain _      []                  = CertificateUsageReject (CertificateRejectOther "empty list")
